package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.espec.util.LimitedLiveLog;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.application.Platform;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class LimitedLiveLogPanel extends VBox implements LimitedLiveLog.ChangeListener {
    @Nonnull private final LimitedLiveLog liveLog;

    @Nonnull private final TextArea textArea;
    @Nonnull private final ProgressIndicator progress;

    public LimitedLiveLogPanel(@Nonnull LimitedLiveLog liveLog) {
        this.liveLog = liveLog;

        this.textArea = new TextArea("Initialising log view...");
        this.progress = new ProgressIndicator(-1);

        liveLog.addChangeListener(this);

        VBox.setVgrow(textArea, Priority.ALWAYS);
        VBox.setVgrow(progress, Priority.NEVER);

        getChildren().addAll(textArea, progress);
        onChanged(liveLog, "");
    }

    @Override
    public void onChanged(@Nonnull LimitedLiveLog source, @Nonnull String addedText) {
        assert source.getText() != null;
        Platform.runLater(() -> {
//          todo scroll to end if now at end  boolean isAtEnd = textArea.getScrollTop() == ??;
            double scrollTop = textArea.getScrollTop();
            textArea.setText(source.getText());
            textArea.setScrollTop(scrollTop);
            progress.setVisible(!source.isEndReached());
        });
    }

    @Override
    public void onEndReached(@Nonnull LimitedLiveLog source) {
        onChanged(source, "");
    }

    public static Dialog createAsDialog(@Nullable Node owner, @Nonnull LimitedLiveLog liveLog) {
        Dialog<String> dialog = new Dialog<>();
        DialogPane dialogPane = new DialogPane();
        dialogPane.setContent(new LimitedLiveLogPanel(liveLog));
        dialogPane.getButtonTypes().setAll(ButtonType.CLOSE);
        dialogPane.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.FILE_TEXT, Color.NAVY));

        if (owner != null && owner.getScene() != null)
            dialog.initOwner(owner.getScene().getWindow());
        dialog.setTitle("Log");
        dialog.setDialogPane(dialogPane);
        dialog.setResizable(true);
        return dialog;
    }
}
